﻿using System;
using System.Data.Common;
using OpenAccessRuntime.Data;
using Telerik.OpenAccess.Data;

namespace TestApplication
{
    public class TimeSpanToFloatConverter : AdoTypeConverter
    {
        public override bool CreateLiteralSql(ref DataHolder holder)
        {
            if (holder.NoValue || holder.ObjectValue == null)
            {
                holder.ObjectValue = "NULL";

                return false;
            }
            else
            {
                return true;
            }
        }

        public override Type DefaultType
        {
            get 
            {
                return typeof(TimeSpan);
            }
        }

        public override AdoTypeConverter Initialize(IDataColumn user, Type clr, IAdoTypeConverterRegistry registry, bool secondary)
        {
            if (clr == typeof(TimeSpan) || clr == typeof(TimeSpan?))
            {
                return base.Initialize(user, clr, registry, secondary);
            }
           
            return AdoTypeConverterUtils.Unsupported(clr, this);
        }

        public override object Read(ref DataHolder holder)
        {
            bool isNull = holder.Reader.IsDBNull(holder.Position);
            holder.NoValue = isNull;
            if (isNull)
            {
                return null;
            }

            double totalMilliseconds = holder.Reader.GetDouble(holder.Position);
            holder.ObjectValue = new TimeSpan((long)totalMilliseconds);

            return holder.ObjectValue;
        }

        public override void Write(ref DataHolder holder)
        {
            DbParameter parameter = holder.Parameter;
            parameter.DbType = System.Data.DbType.Double;

            if (holder.NoValue)
            {
                parameter.Value = DBNull.Value;
            }
            else
            {
                TimeSpan timeSpanValue = (TimeSpan)holder.ObjectValue;
                parameter.Value = timeSpanValue.TotalMilliseconds;
            }
        }
    }
}
